/*
hreorg.c Version 0.1.0  Hash reorganize database
Copyright (C) 2003  dondalah@ripco.com (Dondalah)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to:

	Free Software Foundation, Inc.
	59 Temple Place - Suite 330
	Boston, MA  02111-1307, USA.
*/

/* read the database in physical sequence */
/* one record at a time maintaining position */
/* in hshhdr */
/* hshhdr->ofst   points to the block */

#include <bthash.h>

typedef struct stat statfmt;

int hreorg(hshhdr,totrcds,freespc)
hashfmt *hshhdr;
int totrcds;
double freespc;
   {
   int rslt;
   int lockrslt;
   int rcds;
   int adjadr;
   int esttot;
   int newtot;
   int tmptot;
   int opnerr;
   double dbltot;
   statfmt *stbuf;
   statfmt statbuf;
   int position;
   hashfmt *tmphdr;
   unsigned char *rndSeed;
   unsigned char *p,*q;
   unsigned char tmpdbname[32];
   unsigned char tmphdbn[32];
   hshhdr->reason = 0;
   lockrslt = lockf(hshhdr->hndl,F_TEST,0);
   if (lockrslt < 0)
      {
      return(ISRTLOCK3);
      } /* can't reorg when file is locked */

   lockrslt = lockf(hshhdr->hndl,F_TLOCK,0);
   if (lockrslt < 0)
      {
      return(ISRTLOCK4);
      } /* can't reorg when file is locked */

   stbuf = (statfmt *) &statbuf;
   rslt = fstat(hshhdr->hndl, stbuf);
   if (rslt < 0)
      {
      fprintf(stderr,"hreorg: stat error on %s\n",
	 hshhdr->dbname);
      perror("stat error");
      hshhdr->reason = SYS_ERR + errno;
      return(SYS_ERR + errno);
      } /* stat err */

   adjadr = stbuf->st_size - HDRLEN;
   rcds   = adjadr / hshhdr->rcdnode;

   if (totrcds > 0
      && totrcds != hprime(totrcds))
      {
      fprintf(stderr,"hreorg: total "
	 "records %d is not prime\n",
	 totrcds);
      hshhdr->reason = HASH_ERR + 2402;
      return(HASH_ERR + 2402);
      } /* totrcds not prime */

   newtot = totrcds;

   if (!totrcds)
      {
      dbltot = (double) rcds
	 * (1.0 + (freespc / 100.0));
      esttot = (int) (dbltot + 0.999999);
      newtot = hprime(esttot);
      tmptot = esttot;
      while (newtot < esttot)
	 {
	 tmptot += 2;
	 newtot = hprime(tmptot);
	 } /* get smallest prime larger than rcds */
      } /* calculate new totrcds */
   tmphdr = (hashfmt *) hopent(newtot,
      hshhdr->rcdlen,
      hshhdr->keylen,
      hshhdr->keyofst,
      &opnerr,
      NULL);         /* default seed */
   if (tmphdr == NULL)
      {
      fprintf(stderr,"hreorg: hopent error "
	 "%d\n", opnerr);
      hshhdr->reason = opnerr;
      return(opnerr);
      } /* hopent err */
   position = SWPFRST;
   rslt = 0;
   while (!rslt)
      {
      rslt = hsweep(hshhdr,position,NULL);
      if (!rslt && !hshhdr->reason)
	 {
	 rslt = hisrt(tmphdr,hshhdr->rcd);
	 if (rslt)
	    {
	    fprintf(stderr,"hreorg: "
	       "hisrt error %d reason %d\n",
	       rslt, tmphdr->reason);
            hshhdr->reason = tmphdr->reason;
            return(tmphdr->reason);
	    } /* insert err */
	 } /* if good sweep */
      position = SWPNEXT;
      } /* for each record in database */
   if (rslt != 1)
      {
      fprintf(stderr,"hreorg: "
         "hsweep error %d reason %d\n",
         rslt, hshhdr->reason);
      return(hshhdr->reason);
      } /* if not end of file from sweep */
   rndSeed = (unsigned char *) rndinit();
   p = (unsigned char *) tmpdbname;
   q = (unsigned char *) p + 8;
   while (p < q)
      {
      *p++ = rndnum(26,rndSeed) + 'A';
      } /* for each char in temp dbname */
   *p = '\0';
   strcat(tmpdbname,".HSH");
   strcpy(tmphdbn,tmphdr->dbname);
   hclose(tmphdr);
   /* rename the database name */
   /* to a temporary name */
   rslt = rename(hshhdr->dbname,tmpdbname);
   if (rslt < 0)
      {
      fprintf(stderr,"btreorg: error renaming %s to %s\n",
	 hshhdr->dbname, tmpdbname);
      perror("Rename error");
      return(SYS_ERR + errno);
      } /* rename failed */
   /* rename the reorganized database */
   /* to the original database name */
   rslt = rename(tmphdbn,hshhdr->dbname);
   if (rslt < 0)
      {
      fprintf(stderr,"btreorg: error renaming %s to %s\n",
	 tmphdbn, hshhdr->dbname);
      perror("Rename error");
      return(SYS_ERR + errno);
      } /* rename failed */
   /* remove the temporary name */
   /* to which the original database */
   /* was renamed */
   rslt = remove(tmpdbname);
   if (rslt < 0)
      {
      fprintf(stderr,"btreorg: error removing %s\n",
	 tmpdbname);
      perror("Remove error");
      return(SYS_ERR + errno);
      } /* remove failed */
   if (hshhdr->lock != LOCK)
      {
      lockrslt = lockf(hshhdr->hndl,F_ULOCK,0);
      if (lockrslt < 0)
         {
         fprintf(stderr,"btreorg: error unlocking "
            "%s after reorganization\n",
            hshhdr->dbname);
         perror("Unlock error");
         hshhdr->reason  = SYS_ERR + errno;
	 return(SYS_ERR + errno);
         }
      } /* if no lock during open, unlock */
   return(0);
   } /* hreorg */
