/*
tmpbt.c Version 0.1.0  Btree build test database
Copyright (C) 2003  dondalah@ripco.com (Dondalah)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to:

	Free Software Foundation, Inc.
	59 Temple Place - Suite 330
	Boston, MA  02111-1307, USA.
*/

/* Usage: tmpbt filename */

#include <bthash.h>

#define MAXLMT 999999

void putstx(pgm)
char *pgm;
   {
   fprintf(stderr,"Usage:   %s "
      "#records\n", pgm);
   fprintf(stderr,"Example: %s 100\n", pgm);
   fprintf(stderr,"#records is 1 - %d\n",
      MAXLMT);
   exit(1);
   } /* putstx */

int main(argc,argv)
int argc;
char **argv;
   {
   int opnerr;
   int i;
   int rslt;
   int lmt;
   btreefmt *bthdr;
   unsigned char *rndSeed;
   unsigned char *p,*q;
   unsigned char str[4096];
   if (argc != 2) putstx(*argv);
   lmt = (int) atoi(*(argv+1));
   if (lmt < 1 || lmt > MAXLMT) putstx(*argv);

   rndSeed = (unsigned char *) rndinit();
   if (rndSeed == NULL)
      {
      fprintf(stderr,"tmpbt: rndinit error\n");
      exit(1);
      } /* rndinit err */

   /* 11 records per node is the second most */
   /* optimal nodesz based on the optimbt program */
   /* 3 levels was chosen rather than 2 in order */
   /* to test splits in the database */
   /* You are welcome to test a any other nodesz */ 
   /* btopent numeric parameters */
   /* nodesz, rcdlen, keylen, keyofst, dletofst */
   bthdr = (btreefmt *) btopent(11,16,8,4,2,&opnerr,rndSeed);
   if (bthdr == NULL)
      {
      fprintf(stderr,"tmpbt: open "
	 "error %d\n", opnerr);
      free(rndSeed);
      exit(1);
      } /* open error */
   /* process starts here */

   rslt = 0;
   i = lmt;
   while (!rslt && (i--))
      {
      p = (unsigned char *) str;
      q = (unsigned char *) p + bthdr->keyofst;
      while (p < q)
	 {
	 *p++ = rndnum(26,rndSeed) + 'a';
	 } /* for each char in pre-key */
      q = (unsigned char *) p + bthdr->keylen;
      while (p < q)
	 {
	 *p++ = rndnum(26,rndSeed) + 'A';
	 } /* for each char in key */
      q = (unsigned char *) str + bthdr->rcdlen;
      while (p < q)
	 {
	 *p++ = rndnum(26,rndSeed) + 'a';
	 } /* for each char in post-key */
      *p = '\0';
      rslt = btisrt(bthdr,ROOT,str);
      if (rslt)
	 {
	 fprintf(stderr,"tmpbt: after btisrt rslt %d\n",
	    rslt);
	 exit(1);
	 } /* btisrt err */
      } /* insert loop */
   rslt = btdbck(bthdr);
   if (rslt || bthdr->reason)
      {
      fprintf(stderr,"tmpbt: btdbck error "
	 "rslt %d reason %d\n",
	 rslt, bthdr->reason);
      exit(1);
      } /* if btdbck err */
   printf("tmpbt: temporary database %s\n",
      bthdr->dbname);
   bthdr->seq = 0;
   bttravrs(bthdr,ROOT);

   /**********************************************************/
   /* piping the traverse will prevent the */
   /* temporary file from being removed */
   btcloset(bthdr);
   return(0);
   } /* main */
